// ButtonSprite.c  [DB] mod v1.1

#include <stdio.h>
#include <string.h>

#include "SWIncludes.h"
#include "ButtonSprite.h"
#define kAreControlsDraggable	true		//Allows user to drag buttons around by holding cmd key down
											//Make it false when you do your final compile so end users can't mess with your interface

#define kGridSize				1 /*25*/	//This creates a 'grid' that your controls stick to when you drag them using the cmd key.  [DB]:set to 1
											//Make it 1 to turn off the grid. ***DONT*** make it 0 or you will get a Div by 0 error.

int								gMouseButton, gDraggable, gKeyControlTracking, gTrackingMouse, gMoveButtonX, gMoveButtonY;
Point							gCursorPt;
CursPtr							gNotOverAButtonCursor, gOverAButtonCursor;
SetButtonCursorProcPtr			gSetButtonCursorProc;
EventRecord						gTheEvent;
Boolean							gButtonsAreActive;		// [DB]
Boolean							gShowDraggedPosition;	// [DB]


	// Call once to set up; the Proc is the routine which has the opportunity to adjust cursor when 
	// mouse is over a button (it receives a boolean which says this) [DB]

void SWGInitButtonSpriteWorld(SetButtonCursorProcPtr SetButtonCursorProc)
{
	gSetButtonCursorProc = SetButtonCursorProc;
	gMouseButton = kCursorUp;
	gDraggable = false;
	GetMouse(&gCursorPt);
	gKeyControlTracking = false;
	gTrackingMouse = false;
	gButtonsAreActive = kAreControlsDraggable;			// [DB]
	gShowDraggedPosition = true;						// [DB]
}


	// Call repeatedly to operate; but not more than once per SW frame [DB]

void SWGUpdateButtonData(void)
{
	if (Button() && gButtonsAreActive)				// [DB] simple inhibitor (also inhibits mouse tracking, see below)
	{
		if (gMouseButton==kCursorUp) gMouseButton=kCursorInitDown; else gMouseButton=kCursorDown;
	}
		else gMouseButton=kCursorUp;
	
	(*gSetButtonCursorProc)(gTrackingMouse);

	if (kAreControlsDraggable)
	{
		if (OSEventAvail(0, &gTheEvent)) {}			/* we dont care if its true or false */

		gDraggable = ((gTheEvent.modifiers & cmdKey) != 0);					// [DB]
		gShowDraggedPosition = ((gTheEvent.modifiers & optionKey) != 0);	// [DB]
	}

	GetMouse(&gCursorPt);
	gKeyControlTracking=false;
	gTrackingMouse=false;
}


	// All drawing functions for this button, and the attached action, are declared in one fell swoop [DB]

void SWGSetUpButtonCallbacks(ButtonSpritePtr srcButtonP, ButtonProcPtr DrawButtonUp, 
							ButtonProcPtr DrawButtonMouseOver, ButtonProcPtr DrawButtonDown, 
							ButtonProcPtr DrawButtonDisabled, MoveProcPtr ButtonAction)
{
	srcButtonP->DrawButtonUp=DrawButtonUp;						// for drawing state 0
	srcButtonP->DrawButtonMouseOver=DrawButtonMouseOver;		// for drawing state 1
	srcButtonP->DrawButtonDown=DrawButtonDown;					// for drawing state 2
	srcButtonP->DrawButtonDisabled=DrawButtonDisabled;			// for drawing state 3
	srcButtonP->ButtonAction=ButtonAction;						// action attached to this button
	srcButtonP->LastButtonState=-1;								// force the button to redraw
}


	// Yep.

void SWGEnableButton(ButtonSpritePtr srcButtonP, int IsButtonEnabled)
{
	srcButtonP->IsButtonEnabled=IsButtonEnabled;
}


	// This is the catch -- allows repositioning the button sprite.  [DB]
	// This is an actual move proc; like the other SW move procs, is called by SW to do a move.
	// If not draggable, may activate the button action.

pascal void ButtonMoveProc(SpritePtr srcSpriteP)
{
	if (srcSpriteP->isVisible)
	{	
		if ((!((ButtonSpritePtr)srcSpriteP)->MenuKeyPressed)&&(!gKeyControlTracking))
		{
			if ((((ButtonSpritePtr)srcSpriteP)->IsButtonEnabled)||(gDraggable))
			{
					// state 0  [DB]
				if (gMouseButton==kCursorUp)
				{
					if (((ButtonSpritePtr)srcSpriteP)->TrackingMouse)
					{
						if (SWIsPointInSprite(srcSpriteP,gCursorPt) && gButtonsAreActive) // Button has been unclicked in active button +[DB]
						{
							if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonMouseOver)!=NULL)
								(*((ButtonSpritePtr)srcSpriteP)->DrawButtonMouseOver)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonMouseOver));	//Call the buttons draw proc for a Mouse Over
							((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonMouseOver;
							gTrackingMouse=true;
							if (!gDraggable) (*((ButtonSpritePtr)srcSpriteP)->ButtonAction)(srcSpriteP);	// Do Button
						}
						else
						{
							if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)!=NULL)
								(*((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonUp));	// mouse outside of button
							((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonUp;
						}
					}
					else
					{
						if (SWIsPointInSprite(srcSpriteP,gCursorPt) && gButtonsAreActive)	// +[DB]
						{
							if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonMouseOver)!=NULL)
								(*((ButtonSpritePtr)srcSpriteP)->DrawButtonMouseOver)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonMouseOver));	// mouse inside of button
							((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonMouseOver;
							gTrackingMouse=true;
						}
						else
						{
							if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)!=NULL)
								(*((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonUp));	// mouse outside of button
							((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonUp;
						}
					}
					((ButtonSpritePtr)srcSpriteP)->TrackingMouse=false;
				}
					// state 1  [DB]
				else if (gMouseButton==kCursorDown)
				{
					if (((ButtonSpritePtr)srcSpriteP)->TrackingMouse)
					{
						if (((SWIsPointInSprite(srcSpriteP,gCursorPt))||(gDraggable)) && gButtonsAreActive) // +[DB]
						{
							if (gDraggable)
							{
								gMoveButtonX=((ButtonSpritePtr)srcSpriteP)->TrackingMouseXInit+(short)(((short)gCursorPt.h-(short)((ButtonSpritePtr)srcSpriteP)->TrackingMouseX));
								gMoveButtonY=((ButtonSpritePtr)srcSpriteP)->TrackingMouseYInit+(short)(((short)gCursorPt.v-(short)((ButtonSpritePtr)srcSpriteP)->TrackingMouseY));
								gMoveButtonX=((gMoveButtonX+(kGridSize/2))/(kGridSize))*(kGridSize);
								gMoveButtonY=((gMoveButtonY+(kGridSize/2))/(kGridSize))*(kGridSize);
								SWMoveSprite(srcSpriteP, gMoveButtonX, gMoveButtonY);
								// 
								// [DB]  Optionally show the sprite's position while it is being dragged
								// systematically save//restore any window settings (no optimization needed)
								if (gShowDraggedPosition)
								{
									char 	str [256];
									UInt16	font;
									UInt16	mode;
									UInt16	size;
									// NOTE: fast, easy and not-so-dirty bamboozling of pascal string
									sprintf (str, " %d %d ", gMoveButtonX, gMoveButtonY);	// include a char for the pascal length byte
									str [0] = strlen (str) - 1;								// suddenly it's compatible with a pascal string
									MoveTo (srcSpriteP->destFrameRect.left + 3, srcSpriteP->destFrameRect.bottom - 3);
									font = qd.thePort->txFont;
									size = qd.thePort->txSize;
									mode = qd.thePort->txMode;
									TextMode (srcCopy);
									TextFont (kFontIDGeneva);
									TextSize (9);
									DrawString ((unsigned char *) str);
									TextMode (mode);
									TextFont (font);
									TextSize (size);
								}
								// 
							}
							if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonDown)!=NULL)
								(*((ButtonSpritePtr)srcSpriteP)->DrawButtonDown)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonMouseDown));	// Button is tracking in active button
							((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonMouseDown;
							gTrackingMouse=true;
						}
						else
						{
							if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonMouseOver)!=NULL)
								(*((ButtonSpritePtr)srcSpriteP)->DrawButtonMouseOver)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonMouseOver));	// Button is tracking out of active button
							((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonMouseOver;
						}
					}
					else
					{
						if (SWIsPointInSprite(srcSpriteP,gCursorPt) && gButtonsAreActive)	// +[DB]
						{
							if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)!=NULL)
								(*((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonUp));	// mouse inside of button (but tracked by a different button) 
							((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonUp;
						}
						else
						{
							if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)!=NULL)
								(*((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonUp));	// mouse outside of button
							((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonUp;
						}
					}
				}
					// state 2  [DB]
				else if (gMouseButton==kCursorInitDown)
				{
					if (SWIsPointInSprite(srcSpriteP,gCursorPt) && gButtonsAreActive)	// +[DB]
					{
						if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonDown)!=NULL)
							(*((ButtonSpritePtr)srcSpriteP)->DrawButtonDown)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonMouseDown));	// mouse init clicked inside of button
						((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonMouseDown;
						((ButtonSpritePtr)srcSpriteP)->TrackingMouse=true;										// start button tracking
						gTrackingMouse=true;
						if (gDraggable)
						{
							((ButtonSpritePtr)srcSpriteP)->TrackingMouseX=gCursorPt.h;								// lets move the control around!
							((ButtonSpritePtr)srcSpriteP)->TrackingMouseY=gCursorPt.v;
							((ButtonSpritePtr)srcSpriteP)->TrackingMouseXInit=srcSpriteP->destFrameRect.left;
							((ButtonSpritePtr)srcSpriteP)->TrackingMouseYInit=srcSpriteP->destFrameRect.top;
						}
					}
					else
					{
						if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)!=NULL)
							(*((ButtonSpritePtr)srcSpriteP)->DrawButtonUp)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonUp));	// mouse init clicked outside of button 
						((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonUp;
					}
				}
			}
				// state 3  [DB]
			else
			{
				if ((((ButtonSpritePtr)srcSpriteP)->DrawButtonDisabled)!=NULL)
					(*((ButtonSpritePtr)srcSpriteP)->DrawButtonDisabled)((ButtonSpritePtr)srcSpriteP, !(((ButtonSpritePtr)srcSpriteP)->LastButtonState==kButtonDisabled));	// this button is dimmed and cannot be clicked on 
				((ButtonSpritePtr)srcSpriteP)->LastButtonState=kButtonDisabled;
			}
		}
	}
}

// end  [DB]
